<?php
include_once("channellist.inc.php");
include_once('../../lib/Homegear/Api.php');
/**
 * Class to share data between the main script and the event thread.
 */
class SharedData extends Threaded
{
    public $scriptId = 0;
    public $peerId = 0;
    public $interval = 60;

    public function run() {}
}

class EventThread extends Thread
{
    private $sharedData;

	public function __construct($sharedData)
	{
		$this->sharedData = $sharedData;
	}

	public function run()
	{
		$hg = new \Homegear\Homegear();
		if($hg->registerThread($this->sharedData->scriptId) === false)
		{
			$hg->log(2, "Could not register thread.");
			return;
		}
		$hg->subscribePeer($this->sharedData->peerId);
		while(!$hg->shuttingDown() && $hg->peerExists($this->sharedData->peerId))
		{
			$result = $hg->pollEvent();
			if($result["TYPE"] == "event" && $result["PEERID"] == $this->sharedData->peerId)
			{
				if($result["VARIABLE"] == "REQUEST")
				{
					$this->sharedData->interval = 0;
					$this->synchronized(function($thread){ $thread->notify(); }, $this);
				}
			}
			else if($result["TYPE"] == "updateDevice" && $result["PEERID"] == $this->sharedData->peerId)
			{
				$this->sharedData->interval = 0;
				$this->synchronized(function($thread){ $thread->notify(); }, $this);
			}
		}
	}
    
    function translate($c, $channellist)
    {
        $result = array();
        for ($i = 0; $i < count($c); $i++)
        {
            $value = $c[$i];
            $chn = $channellist[$i];
            if (isset($chn['type']))
            {
                switch ($chn['type'])
                {
                    case 'fix1':
                        $value /= 10;
                        break;
                    case 'ip':
                        $value = long2ip($value);
                        break;
                    case 'unixtime':
                        $value = new Date('c', $value);
                        break;
                    case 'timestamp':
                        /* do nothing here, used for mysql table creation */
                        break;
                    case 'ignore':
                        continue 2;
                        break;
                    case 'enum':
                        if (isset($chn['enum'][$c[$i]]))
                        {
                            $value = $chn['enum'][$c[$i]];
                        }
                        break;
                }
            }
            // push to array
            $result[$chn['name']] = $value;
        }

        return $result;
    }
}

$peerId = (integer)$argv[0];
$hg = new \Homegear\Homegear();

			$hg->log(2, "XXXXXXXXXXXXXXXXX");
$sharedData = new SharedData();
$sharedData->peerId = $peerId;
$sharedData->scriptId = $hg->getScriptId();
$thread = new EventThread($sharedData);
$thread->start();

$last_state = 0;

while(!$hg->shuttingDown() && $hg->peerExists($peerId))
{
	$config = $hg->getParamset($peerId, 0, "MASTER");
    if($config["IP_ADDRESS"] == "")
    {
    	$hg->log(2, "Warning: No ip address set");
    	$thread->synchronized(function($thread){ $thread->wait(5000000); }, $thread);
    	continue;
    }

    $sharedData->interval = $config["REQUEST_INTERVAL"];
    if($sharedData->interval < 60) $sharedData->interval = 60;

    $host = $config["IP_ADDRESS"];
    $port = $config["PORT"];
    
    // Variablen
    $buffer = 0;
    $values = 0; 
    // connecten
    $socket = socket_create(AF_INET, SOCK_STREAM, 0);
    $connect = socket_connect($socket, $host, $port) || exit("socket_connect fehlgeschlagen"); 

    if ($connect != 1)
    {
        if ($last_state != -1)
        {
            $hg->log(3, "Warning: Could not connect to "+$host+":"+$port);
        }
        $last_state = -1;
        continue;
    }
    else
    {
        $last_state = 0;
    }

    //---------------------------------------------------------------------------	

    // Daten holen
    $msg = pack('N*',3004);
    //printf('msg:%s <br>',$msg);
    $send=socket_write($socket, $msg, 4); //3004 senden
    //printf('Bytes send:%d <br>',$send);

    $msg = pack('N*',0);
    //printf('msg:%s <br>',ord($msg));
    $send=socket_write($socket, $msg, 4); //0 senden 
    //printf('Bytes send:%d <br>',$send);

    socket_recv($socket,$rawData,4,MSG_WAITALL);  // Lesen, sollte 3004 zurückkommen
    $rawData = unpack('N*',$rawData);
    //printf('read:%s <br>',implode($rawData));

    socket_recv($socket,$rawData,4,MSG_WAITALL); // Status
    $rawData = unpack('N*',$rawData);
    //printf('Status:%s <br>',implode($rawData));

    socket_recv($socket,$rawData,4,MSG_WAITALL); // Länge der nachfolgenden Werte
    $rawData = unpack('N*',$rawData);
    //printf('L&auml;nge der nachfolgenden Werte:%s <br>',implode($rawData));

    $valueCount = implode($rawData);
    for ($i = 0; $i < $valueCount; ++$i)//vorwärts
    {
        socket_recv($socket,$InBuff[$i],4,MSG_WAITALL);  // Lesen, sollte 3004 zurückkommen
        $rawData[$i] = implode(unpack('N*',$InBuff[$i]));
        //printf("\r\n%d:%s: %d <br>",$i, $channellist[$i]['name'], $rawData[$i]);
    }
    //socket wieder schliessen 
    socket_close($socket);

    $datapoints = $thread->translate($rawData, $channellist);
    
    $hg->setValue($peerId, 1, "Temperatur_TVL", (double)$datapoints["Temperatur_TVL"]);
    $hg->setValue($peerId, 1, "Temperatur_TRL", (double)$datapoints["Temperatur_TRL"]);
}

$thread->join();